#/*----------------------------------------------------------------------------
# *
# *   Copyright (C) 2021 Antonio Augusto Alves Junior
# *
# *   This file is part of RandomIterator.
# *
# *   RandomIterator is free software: you can redistribute it and/or modify
# *   it under the terms of the GNU General Public License as published by
# *   the Free Software Foundation, either version 3 of the License, or
# *   (at your option) any later version.
# *
# *   RandomIterator is distributed in the hope that it will be useful,
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# *   GNU General Public License for more details.
# *
# *   You should have received a copy of the GNU General Public License
# *   along with RandomIterator.  If not, see <http://www.gnu.org/licenses/>.
# *
# *---------------------------------------------------------------------------*/

#
# FindHydra
#
# This module tries to find the Hydra header files and extrats their version.  It
# sets the following variables.
#
# RandomIterator_FOUND       -  Set ON if Hydra headers are found, otherwise OFF.
#
# RandomIterator_INCLUDE_DIR -  Include directory for hydra header files.  (All header
#                       files will actually be in the hydra subdirectory.)
# RandomIterator_VERSION     -  Version of hydra in the form "major.minor.patch".
#

find_path( RandomIterator_INCLUDE_DIR
  PATHS
    ${CMAKE_SOURCE_DIR}
    /usr/include
    /usr/local/include
    ${RandomIterator_ROOT_DIR}
  NAMES random_iterator/RandomIterator.h
  DOC "RandomIterator headers"
  )


if( RandomIterator_INCLUDE_DIR )
  list( REMOVE_DUPLICATES RandomIterator_INCLUDE_DIR )
endif( RandomIterator_INCLUDE_DIR )

# Find hydra version
if (RandomIterator_INCLUDE_DIR)
  file( STRINGS ${RandomIterator_INCLUDE_DIR}/random_iterator/RandomIterator.h
    version
    REGEX "#define RandomIterator_VERSION[ \t]+([0-9x]+)"
    )
  string( REGEX REPLACE
    "#define RandomIterator_VERSION[ \t]+"
    ""
    version
    "${version}"
    )

  string( REGEX MATCH "^[0-9]" major ${version} )
  string( REGEX REPLACE "^${major}00" "" version "${version}" )
  string( REGEX MATCH "^[0-9]" minor ${version} )
  string( REGEX REPLACE "^${minor}0" "" version "${version}" )
  set( RandomIterator_VERSION "${major}.${minor}.${version}")
  set( RandomIterator_MAJOR_VERSION "${major}")
  set( RandomIterator_MINOR_VERSION "${minor}")
endif()

# Check for required components
include( FindPackageHandleStandardArgs )
find_package_handle_standard_args( RandomIterator
  FOUND_VAR     RandomIterator_FOUND
  REQUIRED_VARS RandomIterator_INCLUDE_DIR
  VERSION_VAR   RandomIterator_VERSION
  FAIL_MESSAGE  "RandomIterator could not be found.\
Install RandomIterator and set RandomIterator_ROOT_DIR to point to the headers adding \
 '-DRandomIterator_ROOT_DIR=<path to random_iterator>'in the cmake command."
  )

if(RandomIterator_FOUND)
  set(RandomIterator_INCLUDE_DIRS ${RandomIterator_INCLUDE_DIR})
endif()

mark_as_advanced(RandomIterator_INCLUDE_DIR)
