/*----------------------------------------------------------------------------
 *
 *   Copyright (C) 2021 Antonio Augusto Alves Junior
 *
 *   This file is part of RandomIterator.
 *
 *   RandomIterator is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   RandomIterator is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with RandomIterator.  If not, see <http://www.gnu.org/licenses/>.
 *
 *---------------------------------------------------------------------------*/
/*
 * threefry_practrand.cpp
 *
 *  Created on: 17/03/2021
 *      Author: Antonio Augusto Alves Junior
 */


#include <cstdio>
#include <cstdint>

#include <random_iterator/Stream.hpp>

//command line
#define TCLAP_SETBASE_ZERO 1
#include <tclap/CmdLine.h>

//set a global seed
static const uint64_t default_seed= 0x548c9decbce65295  ;

int main(int argv, char** argc)
{
	uint64_t seed   = default_seed;
	uint32_t stream = 0;

	try {

		TCLAP::CmdLine cmd("Command line arguments for ", '=');

		TCLAP::ValueArg<uint64_t> SeedArg("s", "seed", "RNG seed.", false, default_seed, "uint64_t");
		cmd.add(SeedArg);

		TCLAP::ValueArg<uint32_t> StreamArg("S", "stream", "Stream number.", false, default_seed, "uint32_t") ;
		cmd.add(StreamArg);

		// Parse the argv array.
		cmd.parse(argv, argc);

		// Get the value parsed by each arg.
		seed      = SeedArg.getValue();
        stream    = StreamArg.getValue();
	}
	catch (TCLAP::ArgException &e)  {
		std::cerr << "error: " << e.error() << " for arg " << e.argId()	<< std::endl;
	}

	random_iterator::threefry RNG(seed);

	std::uniform_int_distribution<uint64_t> dist(RNG.min(), RNG.max());

	auto rng_stream = random_iterator::make_stream( dist, RNG, stream);


	constexpr size_t BUFFER_SIZE = 1024 * 1024 / sizeof(uint64_t);
	static uint64_t  buffer[BUFFER_SIZE];

	while (1) {

		for(size_t i = 0; i < BUFFER_SIZE; ++i)
			buffer[i] = rng_stream();

		fwrite((void*) buffer, sizeof(buffer[0]), BUFFER_SIZE, stdout);
	}
}
